#include "mbed.h"
#include "EthernetInterface.h"
#include "arm_math.h"
#include "arm_common_tables.h"

///////////////////////////////////////////////////////////////////////
//
//   frdm_etherClient_netBeans
//   copyright 2016 Thomas P Weldon
//
//tpw 9 jan 2016
//tpw 28 mar 2016
//
///////////////////////////////////////////////////////////////////////
//
//Note: see python code at end
//
//FRDM-K64F ethernet client-server RJ-45 jack example,
//designed to work with a java netBeans socket using java socket interfaces of type:
//       new BufferedReader(new InputStreamReader(clientSocket.getInputStream()))
//       and
//       new PrintWriter(clientSocket.getOutputStream(),true);
//
//runs a client on FRDM-K64F board at hard-coded address CLIENT_ADDRESS below
//a corresponding serve python script is in the comments at the end of this code
//portions/ideas/clips of this code are abstracted from various sources including:
//    mbed.org
//    https://developer.mbed.org/handbook/Socket
//    http://www.prasannatech.net/2008/07/socket-programming-tutorial.html
//    https://developer.mbed.org/platforms/FRDM-K64F/
//    https://developer.mbed.org/questions/906/TCP-Socket-Client-for-Java/
//
//  Procedure:
//       save the python script below onto your laptop
//       plug the frdm-k64f  usb into your laptop
//       before compiling, it is a good idea to right-click the folder and "update all"
//           for some reason, compile twice to eliminate some warnings
//       compile this program and download to your frdm-k64f
//       plug your frdm-k64f ethernet port into your laptop with RJ-45
//       reset your frdm-k64f and the led should turn blue
//       unplug the frdm-k64f  usb from your laptop to turn it off
//       turn off any wifi or ehternet ports connected to routers or internet
//       on a mac OSX laptop, open systemPreference::networks
//             open the ethernet item and note the ethernet address such as 169.254.194.47
//             turn wifi back on and check that the ethernet address (not wifi address) does not change
//             edit the code below to set this as ECHO_SERVER_ADDRESS
//             edit the code below to set this as ECHO_CLIENT_ADDRESS so first 3 numbers match the server
//       re-compile this program with this new address and download to your frdm-k64f
//       reset your frdm-k64f and the led should turn blue
//        run the python script below from a teminal, and you should see the output:
//                  Server Running at  169.254.194.47 :8881   or whatever your server address is
//                  Connected by ('169.254.194.211', 49154)   this is your k64f client address
//       on a mac OSX laptop in a second command terminal, find your teminal by "ls /dev |grep usb"
//           in this second terminal, run your tty program such as "screen /dev/tty.usbmodem1412 9600"
//       reset your frdm-k64f and the led should turn blue and tty output should appear
//       open a second terminal window and run the python script from command line
//       reset your frdm-k64f and the led should turn blue and tty output should appear
//       in a few seconds, the client-side terminal (the "screen" tty output) should show success such as:
//            Connected to Server at 169.254.194.47
//            Waiting for message from server
//       the server-side terminal python-script output should show success such as:
//             Server Running at  169.254.194.47 :8881
//             Server connected by client= ('169.254.194.211', 49153)
//             Type q to Quit or type a message to send
//
//       during the process above, the FRDM-K64F LEDs give status info:
//              blue led color means the k64f is waiting to establish connection
//              green led color means the k64f has establish connection
//              red led color means the k64f has quit/closed the connection
//
//       if you run into trouble on mac yosemite system, try SystemPreferences
//            and in the networks item, check that the address of your
//                ethernet port matches the initial python printout when it started
//                and that it did not accidently attach to your wifi port
//                turn off any wifi or ethernet ports connected to routers or internet
//
//  to learn more about tty terminal use, see https://developer.mbed.org/handbook/SerialPC
//  to see terminal printout on a macbook/yosemite system, connect your frdm=k64f board,
//       open a terminal, look for the tty port, and run screen as follows
//             ls /dev |grep us
//             screen /dev/tty.usbmodem1412 9600
//       using the appropraite tty.usbmodem14xx listed for your machine
//
///////////////////////////////////////////////////////////////////////

DigitalOut pinD0(D0);
DigitalOut grnled(LED_GREEN);
DigitalOut bluled(LED_BLUE);
DigitalOut redled(LED_RED);
Serial pc(USBTX, USBRX); //default 9600 baud, N-8-1 
AnalogOut  dac0out(DAC0_OUT);
AnalogIn  adc0in(A0); //A0 = PTB2
Ticker timer1;

const int RX_BUF_LEN=2048; //socket receive buffer length
const int TX_BUF_LEN=2048; //socket transmit buffer length

//const char* ECHO_SERVER_ADDRESS = "169.254.188.202";
const char* ECHO_SERVER_ADDRESS = "169.254.194.47";
const char* CLIENT_ADDRESS = "169.254.194.211";

const int ECHO_SERVER_PORT = 8881;
const char* SERVER_MASK = "255.255.255.0";  // Mask
//const char*  SERVER_GATEWAY  = "192.168.1.1";    //Gateway
const char*  CLIENT_GATEWAY  = CLIENT_ADDRESS;    //Gateway

#define TEST_LENGTH_SAMPLES 16  //WARNING: only choose power of 2 from 16 to 4096
volatile uint32_t fftSize = TEST_LENGTH_SAMPLES ;
volatile uint32_t ifftFlag = 0;
volatile uint32_t doBitReverse = 1;
volatile uint32_t refIndex = 213, testIndex = 0;
volatile float32_t xx=0, yy=0;
volatile int adcCount=0;
volatile float32_t adcDat[TEST_LENGTH_SAMPLES*2];


void myDacAdc()// this code will run off a timer interrupt writing a simple triangle signal from DAC to ADC
  {
    volatile uint32_t mask16=1<<16; //will write a timer to a single digital pin, DO arduino=PTC16
    volatile uint32_t dat1=(PTC->PDOR) | mask16; //set Port Data Output Register
    volatile uint32_t dat0=(PTC->PDOR) & (!mask16); //clear
    volatile uint16_t tpAdc0in=1,yout=0;
    extern volatile float32_t  xx;
    uint32_t dacOut=0;
        (PTC->PDOR)=dat0; //clear    these 4 code lines send a pulse whennever this code runs 
        (PTC->PDOR)=dat1; //set    
        (PTC->PDOR)=dat1; //set
        (PTC->PDOR)=dat0; //clear
        dacOut=((adcCount & 0x07)<<9);
        //dacOut=(uint16_t)0x0800;
        DAC0->DAT[0].DATL = (uint8_t)((uint16_t)(dacOut)      & 0xFF); //low 8 bits of DAC
        DAC0->DAT[0].DATH = (uint8_t)(((uint16_t)(dacOut) >> 8) & 0x0F); //high 4 bits of 12-bit DAC
       //DAC0->DAT[0].DATH =(uint8_t)(0x08);
       tpAdc0in=adc0in.read_u16();
       xx=(float)((int)tpAdc0in-0x8000);
        if(adcCount<TEST_LENGTH_SAMPLES)
            {adcDat[adcCount*2]=xx;   //save data
            adcDat[adcCount*2+1]=0.0f;
            }
        adcCount=adcCount+1;
}

int main()
{

    char rxbuf[RX_BUF_LEN];  //socket receive buffer
    char txbuf[TX_BUF_LEN];  //socket receive buffer
    int nrx=0; //count of received bytes in a transmission
    float32_t timeDat[TEST_LENGTH_SAMPLES*2]= {   //timeData real and imaginary alternates
        0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,
        1.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,
        0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,
        0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f,0.0f
    };
    float32_t freqDat[TEST_LENGTH_SAMPLES*2]= {   //freqData real and imaginary alternates
        0.0f,0.1f,1.0f,1.1f,2.0f,2.1f,3.0f,3.1f,
        4.0f,4.1f,5.0f,5.1f,6.0f,6.1f,7.0f,7.1f,
        8.0f,8.1f,9.0f,9.1f,10.0f,10.1f,11.0f,11.1f,
        12.0f,12.1f,13.0f,13.1f,14.0f,14.1f,15.0f,15.1f
    };

    bluled=0; // 0=on
    grnled=1; //green off
    redled=1; //off
//  ===========  fft interface follows  =================
        float32_t fftDat[TEST_LENGTH_SAMPLES*2];
        for (int nn=0; nn<TEST_LENGTH_SAMPLES*2; nn++)
             fftDat[nn]=timeDat[nn];
        pc.printf("begin fft  \r\n");
        arm_status status = ARM_MATH_SUCCESS; 
        arm_cfft_radix2_instance_f32 S; 
        ifftFlag = 0;   //forward (ifftFlag=0) or inverse (ifftFlag=1)
        doBitReverse = 1; //Set=1 for output to be in normal order
        fftSize = TEST_LENGTH_SAMPLES ;//WARNING: only choose power of 2 from 16 to 4096
        status = arm_cfft_radix2_init_f32(&S, fftSize,   ifftFlag, doBitReverse);  
        pc.printf("begin fft  \r\n");
        arm_cfft_radix2_f32(&S, fftDat);
        pc.printf("end fft  \r\n");
        for (int nn=0; nn<TEST_LENGTH_SAMPLES; nn++)
        pc.printf("fft dat=%u %f %f \r\n", nn,fftDat[2*nn],fftDat[2*nn+1]);


//  ===========  adc and dac interface follows  =================
//  just write data out of adc into dac
//  !! don forget to use a wire to connect the A0(PTB2) ADC pin to the DAC0_out pin
//    see https://developer.mbed.org/platforms/FRDM-K64F/
//
    if(1) {
        ADC0->SC3 &= ~ADC_SC3_AVGE_MASK;//disable averages
        ADC0->CFG1 &= ~ADC_CFG1_ADLPC_MASK;//high-power mode
        ADC0->CFG1 &= ~0x0063 ; //clears ADICLK and ADIV
        ADC0->CFG1 |= ADC_CFG1_ADIV(0); //divide clock 0=/1, 1=/2, 2=/4, 3=/8
         }
    DAC0->C0 = 0; DAC0->C1 = 0; //reset DAC state
    DAC0->C0 = DAC_C0_DACEN_MASK | DAC_C0_DACSWTRG_MASK| DAC_C0_DACRFS_MASK;

    int nwhile;
    pc.printf("\n Begin DAC and ADC sampling \n");
    adcCount=0;
     timer1.attach(&myDacAdc,12.5e-6f); // calls the myDacAdc function every 12.5 microseconds
   while(adcCount<TEST_LENGTH_SAMPLES)
        {nwhile=nwhile+1; if(nwhile>10000)nwhile=0;} //wait until samples are in
    pc.printf(" End DAC and ADC sampling adcCount=%d\n", adcCount);
    for (int nn=0; nn<TEST_LENGTH_SAMPLES; nn++)
        pc.printf("adc dat=%u %f %f \r\n", nn,adcDat[2*nn],adcDat[2*nn+1]);



//  ===========  ethernet tcp interface follows  =================
    EthernetInterface eth;
    TCPSocketConnection socket;
    bool isconnected=true;

    //eth.init(); //Use DHCP
    //eth.init(ECHO_SERVER_ADDRESS,SERVER_MASK,SERVER_GATEWAY);
  while (true)
{
   pc.printf("\neth.init \n");
   eth.init(CLIENT_ADDRESS,SERVER_MASK,CLIENT_GATEWAY);
    eth.connect();
    pc.printf("\nClient IP Address is %s \n", eth.getIPAddress());
    wait(0.3f);

    // Connect to Server
    while (socket.connect(ECHO_SERVER_ADDRESS, ECHO_SERVER_PORT) < 0) {
        pc.printf("Client waiting: Unable to connect to (%s) on port (%d) eth.getIPAddress= %s\n",
               ECHO_SERVER_ADDRESS, ECHO_SERVER_PORT,eth.getIPAddress() );
        wait(1);
    }
    pc.printf("Connected to Server at %s\n",ECHO_SERVER_ADDRESS);
    bluled=1; // 1=off
    grnled=0;
    redled=1;

//while loop for reading command from server and executing it
   isconnected=true;
    while(isconnected) {
        pc.printf("begin while isconnected \n");
        // Receive message from server
        pc.printf("Waiting for message from server\n");
        nrx = socket.receive(rxbuf, RX_BUF_LEN);
        //rxbuf[nrx] = '\0';
        pc.printf("Received message %d bytes from server=%s ", nrx, rxbuf);
        pc.printf("Received message first byte from server=%c = 0x%X = %d\n,", rxbuf[0],rxbuf[0], (int)rxbuf[0] );
        if(strncmp("tdat",rxbuf,4)==0) {
            //if(true) {
            pc.printf("client time data requested:\n");
            txbuf[0]=(char)0;
            sprintf(txbuf,"%d\n",TEST_LENGTH_SAMPLES);
            pc.printf("txbuf=%s ",txbuf);
            socket.send_all(txbuf, strlen(txbuf) );
            //socket.receive(rxbuf, RX_BUF_LEN);
            for(int nn=0; nn<TEST_LENGTH_SAMPLES; nn++) {
                txbuf[0]=(char)0;
                sprintf(txbuf,"%d,%E,%E\n",nn,timeDat[nn*2],timeDat[nn*2+1]);
                pc.printf("txbuf=%s \n",txbuf);
                socket.send_all(txbuf, strlen(txbuf) );
            }
        } else if(strncmp("fdat",rxbuf,4)==0) {
            //if(true) {
            pc.printf("client freq data requested:\n");
            txbuf[0]=(char)0;
            sprintf(txbuf,"%d\n",TEST_LENGTH_SAMPLES);
            pc.printf("txbuf=%s ",txbuf);
            socket.send_all(txbuf, strlen(txbuf) );
            //socket.receive(rxbuf, RX_BUF_LEN);
            for(int nn=0; nn<TEST_LENGTH_SAMPLES; nn++) {
                txbuf[0]=(char)0;
                sprintf(txbuf,"%d,%E,%E\n",nn,freqDat[nn*2],freqDat[nn*2+1]);
                pc.printf("txbuf=%s \n",txbuf);
                socket.send_all(txbuf, strlen(txbuf) );
            }
        } else        if(strncmp("q",rxbuf,1)==0) {
            pc.printf("client q quitting\n");
            //socket.close();
            //eth.disconnect();
            bluled=1; // 1=off
            grnled=1;
            redled=0;
            wait(2.0f);
            isconnected=false;
        } else {
            pc.printf("Ignoring received message %s\n",rxbuf);
            //strcpy(txbuf,"Client echo:");
            //strcat(txbuf,rxbuf);
            //printf("txbuf=%s \n",txbuf);
            //printf("Sending message first byte from client=%c = 0x%X = %d\n,", txbuf[0],txbuf[0], (int)txbuf[0] );
            //socket.send_all(txbuf, sizeof(txbuf) - 1);
            //socket.send(txbuf, strlen(txbuf) + 1); //this seems to send leading 0 after first send
            //socket.send_all(txbuf, strlen(txbuf) );
        }

    }
        pc.printf("exited while isconnected \n");

    // Clean up
  
    socket.close();
    eth.disconnect();
 
}

}

//python code for a server to run on your laptop
//   note: python script begins at line  "#!/usr/bin/env python"
/*

 #!/usr/bin/env python
# tpw 27 mar 2016
# this works for python 2.7  expect problems in 3.0 and above
import socket
import signal
import sys
import time



def signal_handler(signal, frame):
    print 'You pressed Ctrl+C!'
    server_socket.close()
    client_socket.close()
    sys.exit(0)

signal.signal(signal.SIGINT, signal_handler)
server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
#bind to port 8881
#s.bind(('169.254.196.23', 8881))
server_socket.bind((socket.gethostbyname(socket.gethostname()), 8881))
#s.bind(('', 8881))
server_socket.listen(1)

time.sleep(.300)
print 'Server Running at ', socket.gethostbyname(socket.gethostname()),':8881 \n'
time.sleep(.300)

while True:
    client_socket, addr = server_socket.accept()
    time.sleep(.300)
    print 'Server connected by client=', addr , "\n"

    while True:
        data = raw_input ( "Type q to Quit or type a message to send " )
        if (data == 'q'):
            print "Sending quit to client:" , data + "\n"
            client_socket.send (data)
            client_socket.close()
            server_socket.close()
            sys.exit(0)
            break;
        else:
            print "Sending this to client:" , data + "\n"
            client_socket.send(data)
        data = client_socket.recv(512)
        if ( data == 'q' ):
            client_socket.close()
            server_socket.close()
            sys.exit(0)
            break;
        else:
            print "Received this from client:" , data + "\n"




*/